<?php 

namespace Teknisa\CNC\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Notification;
use Teknisa\CNC\Controllers\TeknisaCrud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DB\StorageProcedure\Param;

use Teknisa\CNC\Services\Cnc01000VendasConciliadasService;
use Teknisa\CNC\Utilities\Util;

class Cnc01000VendasConciliadas extends \Teknisa\CNC\Controllers\TeknisaCrud {
    protected $dataSourceName = 'cnc01000_vendasconciliadas_main';
	protected $parameterBagColumns;
	protected $vendasConciliadasService;
	
	public function __construct (ParameterBag $parameterBag, Cnc01000VendasConciliadasService $vendasConciliadasService, Util $util) {
	    $this->parameterBag = $parameterBag;
        $this->vendasConciliadasService = $vendasConciliadasService;
        $this->util = $util;
	}
	
	public function find (DTO\Request\Filter $request, DTO\Response $response) {
	    try {
	        $conditions = $this->util->getParamsAsArray($request);
			
	        if (!empty( $conditions)){
		        $conditions['TNRESTABELEC']	 = null;
		        $conditions['TNRADQUIRENTE'] = null;
		        $conditions['TNRPRODUTOADQ'] = null;
				
				
				if (empty($conditions['LABELESTABELEC'])) {
		        	$conditions['TNRESTABELEC'] = 'T';
		        	$conditions['NRESTABELEC']  = null;
		        } else {
		        	$conditions['NRESTABELEC']	=
				        	array_reduce($conditions['LABELESTABELEC'], 
						        function($accumulator, $nextElement) {
									return $accumulator.'|'.$nextElement;
				        		}).'|';
		        }
		        unset($conditions['LABELESTABELEC']);
	
		        if (empty($conditions['NMADQUIRENTE'])) {
		        	$conditions['TNRADQUIRENTE'] = 'T';
		        	$conditions['NRADQUIRENTE']  = null;
		        } else {
			        $conditions['NRADQUIRENTE']	=	
			        	array_reduce($conditions['NMADQUIRENTE'], function($accumulator, $nextElement) {
							return $accumulator.'|'.$nextElement;
		        		}).'|';
		        }
		        unset($conditions['NMADQUIRENTE']);
		        
		        if (empty($conditions['NMPRODUTOADQ'])) {
		        	$conditions['TNRPRODUTOADQ'] = 'T';
		        	$conditions['NRPRODUTOADQ']  = null;
		        } else {
			        $conditions['NRPRODUTOADQ']	= array_reduce($conditions['NMPRODUTOADQ'], 
			        	function($accumulator, $nextElement) {
							return $accumulator.'|'.$nextElement;
		        		}).'|';
		        }
		        unset($conditions['NMPRODUTOADQ']);
		        
		        $conditions['DTLOTEINI'] = $conditions['DTLOTEANT'][0];
		        $conditions['DTLOTEFIN'] = $conditions['DTLOTEANT'][1];
		        unset($conditions['DTLOTEANT']);
	
				if(!isset($conditions['CDLOTEVENDA'])) {
			        $conditions['CDLOTEVENDA'] = null;
				}
				
				if(!isset($conditions['CDNSU'])) {
			        $conditions['CDNSU'] = null;
				}
	
				$conditions['TIDCONCILIADO']  = null;
				$conditions['TIDSITUACAO']    = null;
				$conditions['TIDFINRECEBIDO'] = null;
				
				if ($conditions['FINRECEBIDO'] === 'T') {
					$conditions['IDFINRECEBIDO'] = null;
					$conditions['TIDFINRECEBIDO'] = 'T';
				}
	
				if ($conditions['STATUSCON'] === 'T') {
					$conditions['IDCONCILIADO'] = null;
					$conditions['TIDCONCILIADO'] = 'T';
				}
	
				if ($conditions['STATUSSIT'] === 'T') {
					$conditions['IDSITUACAO'] = null;
					$conditions['TIDSITUACAO'] = 'T';
				}
				
				
		        $dataSet = $this->vendasConciliadasService->getLoteVenda($conditions);
		    }
	        else{
	        	$dataSet =[];
	        }
	        $response->addDataSet(new DataSet('cnc01000_vendasconciliadas_main', $dataSet));
	    } catch (\Exception $e) {
	        $response->setError(new Error($e->getMessage(), $e->getCode()));
	    }
	}
	
	public function findTreeGrid (DTO\Request\Filter $request, DTO\Response $response) {
		try {
			error_reporting(E_ALL);
			$conditions = $this->util->getParamsAsArray($request);

			$conditions['TIDCONCILIADO']	=	null;
			$conditions['TIDSITUACAO']		=	null;
			$conditions['TIDFINRECEBIDO']   =   null;

			if (empty($conditions['IDFINRECEBIDO'])) {
				$conditions['IDFINRECEBIDO'] = null;
				$conditions['TIDFINRECEBIDO'] = 'T';
			}
			
			if (empty($conditions['IDCONCILIADO'])) {
				$conditions['IDCONCILIADO'] = null;
				$conditions['TIDCONCILIADO'] = 'T';
			}

			if (empty($conditions['IDSITUACAO'])) {
				$conditions['IDSITUACAO'] = null;
				$conditions['TIDSITUACAO'] = 'T';
			}
			
	        $response->addDataSet(new DataSet('dataSet', $this->vendasConciliadasService->getParcelasVenda($conditions)));
		} catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
		}
	}

	public function getDivergenceConciliator(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();

            $resultado = $this->vendasConciliadasService->getDivergenceConciliator($params);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('Cnc01000_DivergenciaConciliacao', $resultado));

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getPaymentConciliator(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();

            $resultado = $this->vendasConciliadasService->getPaymentConciliator($params);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('Cnc01000_DivergenciaPagamento', $resultado));

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getAdjustmentConciliator(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();

            $resultado = $this->vendasConciliadasService->getAdjustmentConciliator($params);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('Cnc01000_CorrecaoPagamento', $resultado));

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
}