<?php

namespace Teknisa\CNC\Factories;

use Doctrine\ORM\EntityManager;

class Cnc02800DataInicioIntegracaoFactory {
    protected $connection;

    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }

    public function findDataOperador ($params) {
        $sql = "SELECT CTR.CDOPERULTATU, CTR.DTINICIOINT, OP.NMOPERADOR, CTR.DTULTATU
                    FROM CTRLDTFINCNC CTR
                        LEFT JOIN OPERADOR OP
                        ON CTR.CDOPERULTATU = OP.CDOPERADOR
                        AND CTR.NRORG = OP.NRORG
                    WHERE CTR.NRORG = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function findDataOperadorFilial ($params) {
        $sql = "SELECT F.CDFILIAL, F.NMFILIAL, CTR.CDOPERULTATU, CTR.DTINICIOINT, CTR.DTULTATU, OP.NMOPERADOR
                FROM FILIAL F 
                    LEFT JOIN CTRLDTFINCNCFIL CTR
                           ON F.CDFILIAL = CTR.CDFILIAL
                    LEFT JOIN OPERADOR OP
                           ON CTR.CDOPERULTATU = OP.CDOPERADOR
                WHERE F.NRORG = :NRORG
                ORDER BY CDFILIAL";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    public function updateDataOperador($params) {
        $sql = "UPDATE CTRLDTFINCNC
                  SET DTINICIOINT     = :DTINICIOINT,
                      CDOPERULTATU    = :CDOPERULTATU,
                      NRORGULTATU     = :NRORGULTATU,
                      DTULTATU        = :DTULTATU
                WHERE NRORG           = :NRORG";
                  
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function updateDataOperadorFilial($params) {
        $sql = "UPDATE CTRLDTFINCNCFIL
                  SET DTINICIOINT     = :DTINICIOINT,
                      CDFILIAL        = :CDFILIAL,
                      CDOPERULTATU    = :CDOPERULTATU,
                      NRORGULTATU     = :NRORGULTATU,
                      DTULTATU        = :DTULTATU
                WHERE NRORG           = :NRORG
                  AND CDFILIAL        = :CDFILIAL";
                  
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function updateDataOperadorInclude($params) {
        $sql = "UPDATE CTRLDTFINCNC
                  SET DTINICIOINT     = :DTINICIOINT,
                      CDOPERULTATU    = :CDOPERULTATU,
                      NRORGULTATU     = :NRORGULTATU,
                      DTULTATU        = :DTULTATU,
                      IDATIVO         = :IDATIVO,      
                      DTINCLUSAO      = :DTINCLUSAO,     
                      NRORGINCLUSAO   = :NRORGINCLUSAO,     
                      CDOPERINCLUSAO  = :CDOPERINCLUSAO   
                WHERE NRORG           = :NRORG";
                  
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function updateDataOperadorIncludeFilial($params) {
        $sql = "UPDATE CTRLDTFINCNCFIL
                  SET CDFILIAL        = :CDFILIAL,
                      DTINICIOINT     = :DTINICIOINT,
                      CDOPERULTATU    = :CDOPERULTATU,
                      NRORGULTATU     = :NRORGULTATU,
                      DTULTATU        = :DTULTATU,
                      IDATIVO         = :IDATIVO,      
                      DTINCLUSAO      = :DTINCLUSAO,     
                      NRORGINCLUSAO   = :NRORGINCLUSAO,     
                      CDOPERINCLUSAO  = :CDOPERINCLUSAO   
                WHERE NRORG           = :NRORG
                  AND CDFILIAL        = :CDFILIAL";
                  
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function insertDataOperador($params) {
        $sql = "INSERT INTO CTRLDTFINCNC
                  (DTINICIOINT, CDOPERULTATU, NRORGULTATU, DTULTATU, IDATIVO, DTINCLUSAO, NRORGINCLUSAO, 
                    CDOPERINCLUSAO, NRORG) 
                VALUES
                  (:DTINICIOINT, :CDOPERULTATU, :NRORGULTATU, :DTULTATU, :IDATIVO, :DTINCLUSAO, :NRORGINCLUSAO, 
                    :CDOPERINCLUSAO, :NRORG)";
                  
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function insertDataOperadorFilial($params) {
        $sql = "INSERT INTO CTRLDTFINCNCFIL
                  (CDFILIAL, DTINICIOINT, CDOPERULTATU, NRORGULTATU, DTULTATU, IDATIVO, DTINCLUSAO, NRORGINCLUSAO, 
                    CDOPERINCLUSAO, NRORG) 
                VALUES
                  (:CDFILIAL, :DTINICIOINT, :CDOPERULTATU, :NRORGULTATU, :DTULTATU, :IDATIVO, :DTINCLUSAO, :NRORGINCLUSAO, 
                    :CDOPERINCLUSAO, :NRORG)";
                  
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function findData($params) {
        $sql = "SELECT CTR.CDOPERULTATU, CTR.DTINICIOINT, OP.NMOPERADOR
                    FROM CTRLDTFINCNC CTR
                        LEFT JOIN OPERADOR OP
                        ON CTR.CDOPERULTATU = OP.CDOPERADOR
                        AND CTR.NRORG = OP.NRORG
                    WHERE CTR.NRORG = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function findDataFilial($params) {
        $sql = "SELECT CTR.CDFILIAL, CTR.CDOPERULTATU, CTR.DTINICIOINT, OP.NMOPERADOR
                    FROM CTRLDTFINCNCFIL CTR
                        LEFT JOIN OPERADOR OP
                        ON CTR.CDOPERULTATU = OP.CDOPERADOR
                        AND CTR.NRORG = OP.NRORG
                    WHERE CTR.NRORG = :NRORG
                      AND CTR.CDFILIAL = :CDFILIAL";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function verifyInclude($params){
        $sql = " SELECT CTR.NRORGINCLUSAO
                    FROM CTRLDTFINCNC CTR
                        LEFT JOIN OPERADOR OP
                        ON CTR.CDOPERULTATU = OP.CDOPERADOR
                        AND CTR.NRORG = OP.NRORG
                    WHERE CTR.NRORG = :NRORG";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function verifyIncludeFilial($params){
        $sql = " SELECT CTR.NRORGINCLUSAO
                    FROM CTRLDTFINCNCFIL CTR
                        LEFT JOIN OPERADOR OP
                        ON CTR.CDOPERULTATU = OP.CDOPERADOR
                        AND CTR.NRORG = OP.NRORG
                    WHERE CTR.NRORG = :NRORG
                      AND CTR.CDFILIAL = :CDFILIAL";
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
}