<?php

namespace WebServiceCall\Call;

use WebServiceCall\Log\OutputStreamLogger;
use WebServiceCall\Repository\FileManager\IManager;
use WebServiceCall\Repository\FileManager\ConfigManager;

class ConfigWS {
    private const EMPTY_POST_PARAMS = "Favor conferir os parâmetros a serem enviados para este serviço. O NRORG não foi informado.";

    private $periodIni;
    private $periodEnd;
    private $configManager;
    
    public function __construct () {
        $this->setConfigManager(new ConfigManager());
        $configData = $this->configManager->getData();
        
         // Define os parâmetros passados via requisição POST.
        $this->setPOSTParams();
        
        // Define os parâmetros para conexão via Proxy.
        $this->setProxyOptions($configData);
            
        // Define o período que o relatório vai verificar.
        $this->setReportPeriod($configData->getDaysToBeConciliated());
    }
    
    private function setProxyOptions($configData) {
        // Configuração das opções de proxy. 
        // Este trecho é opcional, 
        // uma vez que não necessariamente a conexão terá um proxy.
        $options = [
            'proxy_host' => $configData->getProxyHost(),
            'proxy_port' => $configData->getProxyPort(),
            'proxy_login' => $configData->getProxyLogin(),
            'proxy_password' => $configData->getProxyPassword(),
            'keep_alive' => true,
            'stream_context' => stream_context_create(array(
                'ssl' => array(
                    'verify_peer' => false,
                    'verify_peer_name' => false,
                    'allow_self_signed' => true
                )
            ))
        ];
    }
    
    private function setConfigManager (IManager $manager) {
        $this->configManager = $manager;
    }
    
    private function setPOSTParams() {
        $this->nrorg = $_POST['NRORG'];
        OutputStreamLogger::log("O NRORG informado é: " . $this->nrorg . PHP_EOL);
    }
    
    private function setReportPeriod($daysToBeConciliated) {
        $this->periodIni = date('d/m/Y', strtotime("-" . $daysToBeConciliated . " days"));
        $this->periodEnd = date('d/m/Y', strtotime("-1 days"));
    }
    
    public function getInitialDate() {
        return $this->periodIni;
    }
    
    public function getFinalDate() {
        return $this->periodEnd;
    }
}