<?php 

namespace WebServiceCall\Repository\DB;

class Sales extends ABaseRepo {
    
    public function insertSale($sale) {
        $params = [];

        foreach ($sale as $key => $value) {
            $params[$key] = $value;
        }

        // Gera NRRETORIGVENDA para a linha atual
        $nrOrigVenda = $this->conn->getNextTableId('RETORIGVENDA');
        $params[':NRRETORIGVENDA'] = (int)$nrOrigVenda;
        
        return $this->conn->executeQuery(self::INSERT_SALES, $params, [], true);
    }
    
    /*
     * Verifica se na tabela RETORIGVENDA, já está inserido o registro em questão utilizando a data da venda, o NSU e o 
     * código da autorização, como parâmetros para busca da linha.
     *
     * @param nsu
     * @param autorizacao
     * @param dtvenda
     *
     * @return bool: true se a o select não encontrar nenhuma linha com os dados da filtragem, false caso contrário.
     */
    public function isSaleInserted($nsu, $autorizacao, $dtvenda) {
        $params = array(
            ':CDNSU' => $nsu,
            ':NRAUTORIZACAO' => $autorizacao,
            ':DTVENDA' => $dtvenda,
            ':NRORG' => $this->conn->getNRORG(),
        );
        
        $linhas = $this->conn->fetchRow(self::GET_INSERTED_SALES, $params, [], true);
        
        return intval($linhas) === 0;
    }
    
    private const GET_INSERTED_SALES = "
        SELECT COUNT(*) LINHAS 
          FROM RETORIGVENDA 
         WHERE CDNSU = ':CDNSU'
           AND NRAUTORIZACAO = ':NRAUTORIZACAO'
           AND DTVENDA = TO_DATE(':DTVENDA', 'DD/MM/YYYY')
           AND NRORG = ':NRORG'
    ";

    private const INSERT_SALES = "
        INSERT INTO RETORIGVENDA (
            NRRETORIGVENDA, 
            NRCTRLACESWS, 
            NRESTABELEC, 
            NMADQUIRENTE,
            NRADQUIRENTE,
            CDPRODUTOADQ,
            NMPRODUTOADQ,
            NRPRODUTOADQ,
            IDMODALIDADE,
            DSMODALIDADE,
            DTVENDA,
            DTCREDITO,
            CDLOTEVENDA, 
            NRAUTORIZACAO, 
            NRCARTAOTRUNC, 
            CDNSU,
            VRBRUTVENDA, 
            VRTAXAADM, 
            VRCOBRADOTAXA, 
            VRLIQVENDA, 
            NRTERMINAL, 
            CDBANCO,
            CDAGENCIA,
            CDCONTCORR,
            DTHORATRANS,
            NRPARCELA,
            NRQTPARCELA,
            IDSTATUS,
            IDREJEITADO,
            IDCANCELADO,
            DTPGTO,
            VRLIQPGTO,
            NRORG,
            IDATIVO,
            DTINCLUSAO,
            DTULTATU,
            NRORGINCLUSAO
        ) VALUES (
            ':NRRETORIGVENDA',
            ':NRCTRLACESWS',
            ':Estabelecimento',
            UPPER(':Adquirente'),
            ':NRADQUIRENTE',
            ':CodigoProduto',
            ':DescricaoProduto',
            ':NRPRODUTOADQ',
            ':CodigoModalidade',
            ':DescricaoModalidade',
            TO_DATE(':DataVenda', 'DD/MM/YYYY'),
            TO_DATE(':DataPrevistaPagto', 'DD/MM/YYYY'),
            ':ResumoVenda',
            ':Autorizacao',
            ':Cartao',
            ':Nsu',
            TO_NUMBER(':ValorBruto'),
            TO_NUMBER(':PercentualTaxa'),
            TO_NUMBER(':ValorTaxa'),
            TO_NUMBER(':ValorLiquido'),
            ':Terminal',
            ':Banco',
            ':Agencia',
            ':Conta',
            TO_DATE(':HoraTransacao', 'DD/MM/YYYY HH24:MI:SS'),
            ':Parcela',
            ':TotalParcelas',
            ':Status',
            ':Rejeitado',
            ':Cancelado',
            TO_DATE(':DataPagamento', 'DD/MM/YYYY'),
            TO_NUMBER(':ValorPago'),
            TO_NUMBER(':NRORG'),
            ':IDATIVO', 
            TO_DATE(':DTINCLUSAO', 'DD/MM/YYYY HH24:MI:SS'), 
            TO_DATE(':DTULTATU', 'DD/MM/YYYY HH24:MI:SS'), 
            TO_NUMBER(':NRORG')
        )";
}