<?php
namespace Teknisa\CNC\Utilities;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\FilterCriteria;

use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class UtilCNC extends \Zeedhi\Framework\Controller\Simple {

	protected $instanceManager;
    protected $entityManager;
    protected static function getSalt() {
        return str_pad("zeedhi", 32, "\0");
    }

	public function __construct(\Zeedhi\Framework\DependencyInjection\InstanceManager $instanceManager, \Doctrine\ORM\EntityManager $entityManager) {
		$this->instanceManager = $instanceManager;
        $this->entityManager = $entityManager;
       
	    
	}	

    public function getEnvironment() {
        return $this->instanceManager->getService('environment');
    }


    /**
     * Prepares an SQL statement.
     *
     * @param string $query The Query.
     * @param string $arrName The variable to be replaced.
     * @param array  $arrValue The array of things to be placed on string.
     * @return string The prepared Query.
     *
     */	
	public function putArrayOnQuery($query, $arrName, $arrValue){
        for ($i=0; $i < sizeof($arrValue); $i++) {
            if(key($arrValue) === 0){
                $arrValue[$i] = is_array($arrValue[$i]) ? $arrValue[$i]["_ALL"] : $arrValue[$i];
            }
            if(end($arrValue) == $arrValue[$i]){
                $arrValue[$i] = $arrValue[$i];
            }
            $arrValue[$i] = $arrValue[$i];
        }
        $stample = implode($arrValue);
        $query = preg_replace('/'.$arrName.'/', $stample, $query, 4);
        return $query;
    }

	public function testConnection(Request\Filter $request, Response $response) {
        try {
			$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('UtilitiesTestConnectionRepo', array(array('nothing' => 'nothing'))));
        } catch (\Exception $e) {            
            $response->addMessage(new Message($e->getMessage()));
        }
    }

    public function cleanSpecialChar($string) {
       $string = str_replace('-','',$string); 
       return preg_replace('/[^A-Za-z0-9\-]/', '', $string); // Removes special chars.
    }

    public function checkVersion($frontVersion){
        $backVersion = $this->instanceManager->getParameter('backVersion');

        if ($backVersion === $frontVersion) {
        	$versionOk = true;	
        } else {
        	$versionOk = false;	
        } 

        return array(
            'versionOk' => $versionOk,
            'frontVersion' => $frontVersion,
            'backVersion' => $backVersion
        );
    }    

    public function compararMenorData($dataAtual, $data){
        $ano1 = substr($dataAtual, 4,8);
        $mes1 = substr($dataAtual, 2,2);
        $dia1 = substr($dataAtual, 0,2);

        $ano2 = substr($data, 4,8);
        $mes2 = substr($data, 2,2);
        $dia2 = substr($data, 0,2);

        if($ano2 < $ano1){
            return false;
        }else if($ano2 == $ano1 && $mes2 < $mes1){
            return false;
        }else if($ano2 == $ano1 && $mes2 == $mes1 && $dia2 < $dia1){
            return false;
        }else{
            return true;
        }
    }

    public function replaceParams($params){
        for($i = 0; $i < count($params); ++$i) {            
           if($params[$i]["value"] == '='){
             $params[$i]["value"] = '';
           }
        }
        return $params;
    }

    public static function encrypt($text, $salt = 'setforms')
    {        
        return md5('odhen'.$text.'7833');
    }

    public static function oldEncrypt($text)
    {
        return trim(
            base64_encode(
                mcrypt_encrypt(
                    MCRYPT_RIJNDAEL_256,
                    self::getSalt(),
                    $text,
                    MCRYPT_MODE_ECB,
                    mcrypt_create_iv(mcrypt_get_iv_size(MCRYPT_RIJNDAEL_256, MCRYPT_MODE_ECB), MCRYPT_RAND)
                )
            )
        );
    }


    /**
     * Decrypt.
     *
     * @param string $text Text.
     *
     * @return string
     */
    public static function decrypt($text)
    {
        return trim(
            mcrypt_decrypt(
                MCRYPT_RIJNDAEL_256,
                self::getSalt(),
                base64_decode($text),
                MCRYPT_MODE_ECB,
                mcrypt_create_iv(mcrypt_get_iv_size(MCRYPT_RIJNDAEL_256, MCRYPT_MODE_ECB), MCRYPT_RAND)
            )
        );
    }

    public function formataDataBr($data){
        return implode(preg_match("~\/~", $data) == 0 ? "/" : "-", array_reverse(explode(preg_match("~\/~", $data) == 0 ? "-" : "/", $data)));
    }


    public function formataDinheiro($num){
        return 'R$' . number_format($num, 2, ',', '.');
    }
    public function formataPorcentagem($num){
        return  $num . '%';
    }

    public function newCode($counterName, $size){
        $nrseq = null;
        $connection = $this->entityManager->getConnection();
        $storedProcedure = new StoredProcedure($connection, 'NOVO_CODIGO');
        $storedProcedure->addParam(new Param('P_CONTADOR', Param::PARAM_INPUT, $counterName, Param::PARAM_TYPE_STR, strlen($counterName)));
        $storedProcedure->addParam(new Param('P_SEQUENCIAL', Param::PARAM_OUTPUT, $nrseq, Param::PARAM_TYPE_INT, 20));
        $result = $storedProcedure->execute();
        return substr($result['P_SEQUENCIAL'], strlen($result['P_SEQUENCIAL'])-$size);
    }
    
    public function newCodeNT($counterName, $size){
        $nrseq = null;
        $connection = $this->entityManager->getConnection();
        $storedProcedure = new StoredProcedure($connection, 'NOVO_CODIGO_NT');
        $storedProcedure->addParam(new Param('P_CONTADOR', Param::PARAM_INPUT, $counterName, Param::PARAM_TYPE_STR, strlen($counterName)));
        $storedProcedure->addParam(new Param('P_SEQUENCIAL', Param::PARAM_OUTPUT, $nrseq, Param::PARAM_TYPE_INT, 20));
        $result = $storedProcedure->execute();
        return substr($result['P_SEQUENCIAL'], strlen($result['P_SEQUENCIAL'])-$size);
    }
    
    public function procedAtualizaEstoque($CDFILIAL, $NRLANCESTQ){
        $connection = $this->entityManager->getConnection();
        $storedProcedure = new StoredProcedure($connection, 'ATUALIZA_ESTOQUE_MANAGE');

        $dataAtual = date('d:m:Y');

        $storedProcedure->addParam($this->buildInputParam('P_CDFILILANC', $CDFILIAL));
        $storedProcedure->addParam($this->buildInputParam('V_DTINICIO', $dataAtual));
        $storedProcedure->addParam($this->buildInputParam('V_DTFIM', $dataAtual));
        $storedProcedure->addParam($this->buildInputParam('P_CDFILIAL', $CDFILIAL));
        $storedProcedure->addParam($this->buildInputParam('P_NRLANCESTQ', $NRLANCESTQ));
        
        $result = $storedProcedure->execute();
        return $result;
    }

    public function procedInsereItensDeComposicao($CDFILIAL, $NRLANCESTQ, $CDPRODUTO, $QTLANCTOEST, 
                                                $VRUNILANCTO, $TIPOMOVI, $CDALMOXARIFE, $CDLOCALESTOQ){
        
        $connection = $this->entityManager->getConnection();
        $storedProcedure = new StoredProcedure($connection, 'INSERE_SUBPRODMAN');

        $dataAtual = date('d:m:Y');

        $storedProcedure->addParam($this->buildInputParam('P_CDFILIAL', $CDFILIAL));
        $storedProcedure->addParam($this->buildInputParam('P_LANCESTQ', $NRLANCESTQ));
        $storedProcedure->addParam($this->buildInputParam('P_TIPOOPER', '10'));
        $storedProcedure->addParam($this->buildInputParam('P_CDPRODUTO', $CDPRODUTO));
        $storedProcedure->addParam($this->buildInputParam('P_QTITEMPROD', $QTLANCTOEST));
        $storedProcedure->addParam($this->buildInputParam('P_VRTOTAITEM', $VRUNILANCTO * $QTLANCTOEST));
        $storedProcedure->addParam($this->buildInputParam('P_VRDESCITEM', '0'));
        $storedProcedure->addParam($this->buildInputParam('P_VRBRUTITEM', $VRUNILANCTO));
        $storedProcedure->addParam($this->buildInputParam('P_VRCONTITEM', '0'));
        $storedProcedure->addParam($this->buildInputParam('P_VRTOTALANCTO', $VRUNILANCTO * $QTLANCTOEST));
        $storedProcedure->addParam($this->buildInputParam('P_QTDELANCTO', $QTLANCTOEST));
        $storedProcedure->addParam($this->buildInputParam('P_TIPOMOVI', $TIPOMOVI));
        $storedProcedure->addParam($this->buildInputParam('P_INIVALID', $dataAtual));
        $storedProcedure->addParam($this->buildInputParam('P_CDPRODESTO', $CDPRODUTO));
        $storedProcedure->addParam($this->buildInputParam('P_NRPEDIDO', null));
        $storedProcedure->addParam($this->buildInputParam('P_NRITEMPED', null));
        $storedProcedure->addParam($this->buildInputParam('P_IDTPCOMPPROD', 'F'));
        $storedProcedure->addParam($this->buildInputParam('P_CDALMOXARIFE', $CDALMOXARIFE));
        $storedProcedure->addParam($this->buildInputParam('P_CDLOCALESTOQ', $CDLOCALESTOQ));
        
        $result = $storedProcedure->execute();
        return $result;
    }

    private function buildInputParam ($paramName, $paramValue) {
        return new Param($paramName, Param::PARAM_INPUT, $paramValue, Param::PARAM_TYPE_STR, strlen($paramValue));
    }

    public function formataHora4Digitos($horario){
        $hora = substr($horario, 0,2);
        $minutos = substr($horario, 2,5);

        return $hora . ':' . $minutos;
    }

    public function formataHora4DigitosReverso($horario){
        $hora = substr($horario, 0,2);
        $minutos = substr($horario, 3,5);

        return $hora . $minutos;
    }

    public static function nextNDay($dias){
        return date("d-m-Y",mktime(0, 0, 0, date("m"), date("d")+$dias, date("y")));
    }

    public function getParams ($params) {
    	
        $treatedParams = array();
        foreach ($params as $param) {
            
             if (!empty($param['value'])) {
	            if ($param['value'] == '=') {
	                $param['value'] = '';
	            }
            }
            $treatedParams[$param['columnName']] = $param['value'];
        }

        return $treatedParams;
    }

    public function formatDate($date){
        //Formata o dia e o mes com dois algarismos e retorna a data no formato brasileiro
        $date = explode("/",$date);        
        return str_pad($date[0],2,"0",STR_PAD_LEFT) .'/'.str_pad($date[1],2,"0",STR_PAD_LEFT).'/'.$date[2];
    }

    public function directoryCheck($directory){
        
        if ( !is_dir($directory) && ! @mkdir($directory, 0777, true)) {
            throw new \InvalidArgumentException(sprintf(
                'O diretório "%s" não pode ser criado.',
                $directory
            ));
        }

        if ( ! is_writable($directory)) {
            throw new \InvalidArgumentException(sprintf(
                'O diretório "%s" não pode tem permissão de escrita.',
                $directory
            ));
        }
    }

    // Use it to get Zeedhi's Request's objects as array objects
    
    // DataSet  -> array of Zeedhi Rows
    // Row      -> Row  
    // Filter   -> associative array like an Zeedhi Row 
    
    public function getParamsAsArray($obj) {
        if($obj instanceof Request\DataSet) {
            // returns An Array
            return $obj->getDataSet()->getRows();
        } elseif ($obj instanceof Request\Row) {
            return $obj;
        } elseif ($obj instanceof Request\Filter) {
            $arr = array();
            foreach ($obj->getFilterCriteria()->getConditions() as $condition) {
                $arr[$condition['columnName']] = $condition['value'];  
            }
            return $arr;
        }
    }
    
    // Testing
    public function getZeedhiRowAsArray(Request\DataSet $obj) {
        $arr = [];
        foreach ($obj as $key => $value) {
            $arr[$key] = $value;
        }
        return $arr;
        
    }

    public function  moneyToDouble($value){
        if ($value){
            $value = preg_replace("/[^0-9,]/", "", $value);
            $value = str_replace ("," , "." , $value);          
        }
        else {
            $value = 0;
        }
        
        return floatval($value);
    }
    
    public function retrieveValidator ($filter) {
        $bool = array_filter($filter, function($param) {
            return strtoupper($param["columnName"]) == "VALIDATOR";
        });
        $bool = array_shift($bool);
        $bool = $bool["value"] ? true : false;
        return $bool;
    }
    
    public function getParamValueFromFilter ($filter, $paramName) {
        $obj = array_filter($filter, function($param) use ($paramName) {
            return $param["columnName"] === $paramName;
        });
        return empty($obj) ? "" : array_shift($obj)["value"];
    }
    
    public function buildParamValue ($arr) {
        return array_reduce($arr, function($accumulator, $element) {
            return $accumulator."|".$element;
        }, "")."|";
    }
    
    public function buildConditionElementFromArray ($condition, $columnName, $operator) {
        $paramCondition = [];
        if ($condition) {
            $paramCondition["columnName"] = $columnName;
            $paramCondition["operator"]   = $operator;
            $paramCondition["value"]      = $this->buildParamValue($condition["value"]);
        }
        
        return $paramCondition;
    }
    
    public function buildConditionElementFromString ($condition, $columnName, $operator) {
        $paramCondition = [];
        if ($condition) {
            $value = str_replace("%", "|", $condition["value"]);
            $value = str_replace(",", "|", $value);
            
            $paramCondition["columnName"] = $columnName;
            $paramCondition["operator"]   = $operator;
            $paramCondition["value"]      = $value;
        }
        
        return $paramCondition;
    }
    
    public function getParamFromConditions ($conditions, $searchName) {
        $param = array_filter($conditions, function($param) use ($searchName) {
            return $param["columnName"] === $searchName;
        });
        
        return array_shift($param);
    }
    
    public function returnEmptyParam ($columnName, $operator) {
        return array(
            "columnName" => $columnName,
            "operator"   => $operator,
            "value"      => ""
        );
    }
    
    // Returns the names of the conditions that use the operator LIKE
    public function getLikeConditions($request) {
        $conditions = $request->getFilterCriteria()->getConditions();
        $result = array();
        foreach ($conditions as $condition) {
            if($condition['operator'] === 'LIKE_ALL' || $condition['operator'] === 'LIKE_I' || $condition['operator'] === 'MAPPED_LIKE_ALL' )
                array_push($result, $condition);
        }
        return $result;
    }
    
    // Returns a new Zeedhi's FilterCriteria's Request object with the conditions array.
    // Obs: all conditions will have the operator equal ('=') in it.
    public function createNewZeedhiFilterRequestObject($request, $dataSourceName, $conditions) {
        foreach($conditions as $key => $value) {
            $dataSourceFilter[] = $condition = array(
                "columnName" => $key,
                "operator"=> "=",
                "value"=> $value
            );
        }
        
        return new DTO\Request\Filter(
            new FilterCriteria(
                $dataSourceName,
                $dataSourceFilter,
                $request->getFilterCriteria()->getPage(),
                $request->getFilterCriteria()->getPageSize()
            ),
            $request->getMethod(),
            $request->getRoutePath(),
            $request->getUserId()
        );
    }

    // BeforeDate -> String in Brazilian default pattern.
    // AfterDate -> String in Brazilian default pattern.
    // BeforeDate must be equal or come before LastDate.
    
    // Returns:
    // True if dates are equal or if BeforeDate is equal or comes before LastDate
    // False if BeforeDate comes after LastDate
    public function compareDates($beforeDate, $afterDate) {
        $date1 = intval(implode('', array_reverse(explode('/', $beforeDate))));
        $date2 = intval(implode('', array_reverse(explode('/', $afterDate))));
        
        // A data mais recente será maior que a data mais antiga.
        return  $date1 > $date2;
    }
}